@@ -1,6 +1,3 @@
-import os
-from pathlib import Path
-
 from fastapi.testclient import TestClient
 
 from docs_src.request_forms_and_files.tutorial001 import app
@@ -163,32 +160,30 @@ def test_post_body_json():
     assert response.json() == file_and_token_required
 
 
-def test_post_file_no_token(tmpdir):
-    path = os.path.join(tmpdir, "test.txt")
-    with open(path, "wb") as file:
-        file.write(b"<file content>")
+def test_post_file_no_token(tmp_path):
+    path = tmp_path / "test.txt"
+    path.write_bytes(b"<file content>")
 
     client = TestClient(app)
-    response = client.post("/files/", files={"file": open(path, "rb")})
+    with path.open("rb") as file:
+        response = client.post("/files/", files={"file": file})
     assert response.status_code == 422, response.text
     assert response.json() == token_required
 
 
-def test_post_files_and_token(tmpdir):
-    patha = Path(tmpdir) / "test.txt"
-    pathb = Path(tmpdir) / "testb.txt"
+def test_post_files_and_token(tmp_path):
+    patha = tmp_path / "test.txt"
+    pathb = tmp_path / "testb.txt"
     patha.write_text("<file content>")
     pathb.write_text("<file b content>")
 
     client = TestClient(app)
-    response = client.post(
-        "/files/",
-        data={"token": "foo"},
-        files={
-            "file": patha.open("rb"),
-            "fileb": ("testb.txt", pathb.open("rb"), "text/plain"),
-        },
-    )
+    with patha.open("rb") as filea, pathb.open("rb") as fileb:
+        response = client.post(
+            "/files/",
+            data={"token": "foo"},
+            files={"file": filea, "fileb": ("testb.txt", fileb, "text/plain")},
+        )
     assert response.status_code == 200, response.text
     assert response.json() == {
         "file_size": 14,
